"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.AsynchronousValidator = void 0;
const validator_1 = require("./validator");
class AsynchronousValidator {
    constructor(validatedProperties, furtherProperties) {
        this.validatedProperties = validatedProperties;
        this.furtherProperties = furtherProperties;
        this.lastValidationResult = validator_1.ValidationPassed;
    }
    getValidatedProperties() {
        return this.validatedProperties;
    }
    getAdditionalProperties() {
        return this.furtherProperties;
    }
    validate() {
        return __awaiter(this, void 0, void 0, function* () {
            if (!this.needsValidation(this.validatedProperties, this.furtherProperties)) {
                return this.lastValidationResult;
            }
            try {
                const result = yield this.process(this.validatedProperties, this.furtherProperties);
                const msgs = this.getMessagesFromResult(result);
                if (msgs == null || msgs instanceof Array) {
                    this.lastValidationResult = {
                        getMessages: (propertyId) => {
                            return msgs;
                        }
                    };
                }
                else {
                    this.lastValidationResult = {
                        getMessages: (propertyId) => {
                            var _a;
                            return (_a = msgs.get(propertyId)) !== null && _a !== void 0 ? _a : null;
                        }
                    };
                }
                return this.lastValidationResult;
            }
            catch (error) {
                const msg = this.getMessageOnError(error);
                this.lastValidationResult = {
                    getMessages: (propertyId) => {
                        return msg && [msg];
                    }
                };
                return this.lastValidationResult;
            }
        });
    }
    getLastValidationResult() {
        return this.lastValidationResult;
    }
    isAsynchronous() {
        return true;
    }
}
exports.AsynchronousValidator = AsynchronousValidator;
