"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.AsynchronousSingleValidator = void 0;
const validator_1 = require("./validator");
class AsynchronousSingleValidator {
    constructor(property) {
        this.lastValidationResult = validator_1.ValidationPassed;
        this.propertyList = [property];
    }
    getValidatedProperties() {
        return this.propertyList;
    }
    getAdditionalProperties() {
        return [];
    }
    validate() {
        return __awaiter(this, void 0, void 0, function* () {
            const currentValue = this.propertyList[0].getValue();
            if (currentValue !== this.lastValue && this.lastValue === undefined) {
                return this.lastValidationResult;
            }
            try {
                const result = yield this.process(currentValue);
                const msgs = this.getMessagesFromResult(result);
                this.lastValidationResult = {
                    getMessages: (propertyId) => {
                        return msgs;
                    }
                };
                return this.lastValidationResult;
            }
            catch (error) {
                const msg = this.getMessageOnError(error);
                this.lastValidationResult = {
                    getMessages: (propertyId) => {
                        return msg && [msg];
                    }
                };
                return this.lastValidationResult;
            }
        });
    }
    getLastValidationResult() {
        return this.lastValidationResult;
    }
    isAsynchronous() {
        return true;
    }
}
exports.AsynchronousSingleValidator = AsynchronousSingleValidator;
