import { PropertyId } from "../../properties/property-id";
import { PropertyScalar } from "../../properties/property-scalar";
import { PropertyScalarImpl } from "../../properties/property-scalar-impl";
import { AbstractProperty } from "../../properties/abstract-property";
import { ValueConverter } from "../../value-converter/value-converter";
import { Choice } from "../../properties/choice";
import { ValueProvider } from "../../provider/value-provider/value-provider";
import { PropertyScalarRuleBinding } from "./property-scalar-rule-binding";
import { PropertyTemplate } from "../../properties/factory/property-template";
import { EmptyValueFcn } from "../../provider/value-provider/empty-value-fcn";
import { BackpressureConfig } from "../../properties/backpressure/backpressure-config";
import { PropertyScalarWithChoices } from "../../properties/property-scalar-with-choices";
import { ListIndex } from "../../properties/factory/list-index";
import { SiblingAccess } from "../../provider/list-provider/sibling-access";
export declare class PropertyScalarBuilder {
    private readonly propertyScalar;
    private readonly bindPropertScalar;
    private readonly defaultEmptyChoice;
    constructor(propertyScalar: <T>(id: PropertyId, provider: ValueProvider<T>, emptyValueFcn: EmptyValueFcn<T>, converter: ValueConverter<T>, dependencies?: AbstractProperty<unknown>[], initialValue?: T | null, backpressureConfig?: BackpressureConfig, ownedProperties?: AbstractProperty<unknown>[]) => PropertyScalarImpl<T>, bindPropertScalar: <T>(prop: PropertyScalar<T>) => PropertyScalarRuleBinding<T>, defaultEmptyChoice: Choice<any> | undefined);
    template<T>(id: string, factory: (scalarBuilder: PropertyScalarBuilder, id: PropertyId, index?: ListIndex, siblingAccess?: SiblingAccess<PropertyScalar<T>, T>) => PropertyScalar<T>): PropertyTemplate<PropertyScalar<T>, T>;
    isLike<T>(template: PropertyScalar<T>, valueProvider?: ValueProvider<T>): PropertyScalar<T>;
    simpleProperty<T>(id: PropertyId, valueConverter: ValueConverter<T>, emptyValueFcn?: EmptyValueFcn<T>): PropertyScalar<T>;
    stringProperty(id: PropertyId, options?: {
        valueConverter?: ValueConverter<string>;
        initialValue?: string | null;
    }): PropertyScalar<string>;
    numberProperty(id: PropertyId, options?: {
        valueConverter?: ValueConverter<number>;
        zeroIsConsideredAsEmpty?: boolean;
        initialValue?: number | null;
    }): PropertyScalar<number>;
    booleanProperty(id: PropertyId, options?: {
        valueConverter?: ValueConverter<boolean>;
        initialValue?: boolean | null;
    }): PropertyScalar<boolean>;
    dateProperty(id: PropertyId, options?: {
        valueConverter?: ValueConverter<Date>;
        initialValue?: Date | null;
    }): PropertyScalar<Date>;
    select: {
        static: <T>(id: PropertyId, choices: Choice<T>[], emptyChoice?: Choice<T> | undefined) => PropertyScalarWithChoices<T, T>;
        derived1: <T_1, TD, D1 extends AbstractProperty<TD>>(id: PropertyId, dependency: D1, derivations: {
            derive: (dep: D1) => Choice<T_1>[];
        }, emptyChoice?: Choice<T_1> | undefined) => PropertyScalarWithChoices<T_1, T_1>;
        asyncDerived1: <T_2, TD_1, D1_1 extends AbstractProperty<TD_1>>(id: PropertyId, dependency: D1_1, derivations: {
            deriveAsync: (dep: D1_1) => Promise<Choice<T_2>[]>;
            backpressureConfig?: BackpressureConfig | undefined;
        }, emptyChoice?: Choice<T_2> | undefined) => PropertyScalarWithChoices<T_2, T_2>;
        withPropertySource: <T_3>(id: PropertyId, choicesSource: PropertyScalar<Choice<T_3>[]>, emptyChoice?: Choice<T_3> | undefined) => PropertyScalarWithChoices<T_3, T_3>;
    };
    typeahead: {
        async: <T>(id: PropertyId, options: {
            fetchChoices: (currentText: string) => Promise<Choice<T>[]>;
            minimumTextLength?: number | undefined;
        }) => PropertyScalarWithChoices<[T | null, string], T>;
        withPropertySource: <T_1>(id: PropertyId, inputSource: PropertyScalar<string>, choicesSource: PropertyScalar<Choice<T_1>[]>) => PropertyScalarWithChoices<[T_1 | null, string], T_1>;
    };
    objectProperty<T, O>(id: PropertyId, obj: O, get: (obj: O) => T | null, set: (obj: O, val: T | null) => void, valueConverter: ValueConverter<T>, emptyValueFcn?: EmptyValueFcn<T>): PropertyScalar<T>;
    objectPathProperty<T, O>(id: PropertyId, obj: O, path: string, valueConverter: ValueConverter<T>, emptyValueFcn?: EmptyValueFcn<T>): PropertyScalar<T>;
    enviromentProperty<T>(id: PropertyId, valueConverter: ValueConverter<T>, value: T | (() => T), emptyValueFcn?: EmptyValueFcn<T>): PropertyScalar<T>;
    derived: {
        sync1: <T, TD, D1 extends AbstractProperty<TD>>(id: PropertyId, valueConverter: ValueConverter<T>, dependency: D1, derivations: {
            derive: (dep: D1) => T | null;
            inverse?: ((dep: D1, val: T | null) => void) | undefined;
        }, emptyValueFcn?: EmptyValueFcn<T>) => PropertyScalar<T>;
        sync2: <T_1, TD_1, D1_1 extends AbstractProperty<TD_1>, D2 extends AbstractProperty<TD_1>>(id: PropertyId, valueConverter: ValueConverter<T_1>, dependency1: D1_1, dependency2: D2, derivations: {
            derive: (dep1: D1_1, dep2: D2) => T_1 | null;
            inverse?: ((dep1: D1_1, dep2: D2, val: T_1 | null) => void) | undefined;
        }, emptyValueFcn?: EmptyValueFcn<T_1>) => PropertyScalar<T_1>;
        async0: <T_2>(id: PropertyId, valueConverter: ValueConverter<T_2>, derivations: {
            deriveAsync: () => Promise<T_2 | null>;
            inverseAsync?: ((val: T_2 | null) => Promise<unknown>) | undefined;
            backpressureConfig?: BackpressureConfig | undefined;
        }, emptyValueFcn?: EmptyValueFcn<T_2>) => PropertyScalar<T_2>;
        async1: <T_3, TD_2, D1_2 extends AbstractProperty<TD_2>>(id: PropertyId, valueConverter: ValueConverter<T_3>, dependency: D1_2, derivations: {
            deriveAsync: (dep: D1_2) => Promise<T_3 | null>;
            inverseAsync?: ((dep: D1_2, val: T_3 | null) => Promise<unknown>) | undefined;
            backpressureConfig?: BackpressureConfig | undefined;
        }, emptyValueFcn?: EmptyValueFcn<T_3>) => PropertyScalar<T_3>;
        async2: <T_4, TD_3, D1_3 extends AbstractProperty<TD_3>, D2_1 extends AbstractProperty<TD_3>>(id: PropertyId, valueConverter: ValueConverter<T_4>, dependency1: D1_3, dependency2: D2_1, derivations: {
            deriveAsync: (dep: D1_3, dep2: D2_1) => Promise<T_4 | null>;
            inverseAsync?: ((dep: D1_3, dep2: D2_1, val: T_4 | null) => Promise<unknown>) | undefined;
            backpressureConfig?: BackpressureConfig | undefined;
        }, emptyValueFcn?: EmptyValueFcn<T_4>) => PropertyScalar<T_4>;
    };
    bind<T>(prop: PropertyScalar<T>): PropertyScalarRuleBinding<T>;
    private invFcn;
    private asycInvFcn;
}
