import { alwaysAssertThat } from "../../util/assertions/assertions";
import { A } from "../../attributes/attribute-id";
export var TextInterpreter;
(function (TextInterpreter) {
    TextInterpreter[TextInterpreter["Plain"] = 0] = "Plain";
    TextInterpreter[TextInterpreter["Markdown"] = 1] = "Markdown";
    TextInterpreter[TextInterpreter["Html"] = 2] = "Html";
})(TextInterpreter || (TextInterpreter = {}));
export class PropertyScalarRuleBinding {
    constructor(property, notEmptyIfRequiredValidator, addDependencies) {
        this.notEmptyIfRequiredValidator = notEmptyIfRequiredValidator;
        this.addDependencies = addDependencies;
        this.property = property;
    }
    addScalarValidator(validator) {
        this.property.addScalarValidator(validator);
        return this;
    }
    addValidator(validator) {
        this.property.addValidator(validator);
        const dependencies = validator.getValidatedProperties().concat(validator.getAdditionalProperties());
        this.addDependencies(dependencies, this.property, { validation: true });
        return this;
    }
    defineInitialValue(value) {
        alwaysAssertThat(!this.property.isReadOnly(), () => `${this.property.id}: Can not define initial value on read only property`);
        this.property.defineInitialValue(value);
        this.property.setToInitialValue();
        return this;
    }
    define(attribudeId, value) {
        if (value instanceof Function) {
            this.defineAttributeFunction(attribudeId, [], () => value(this.property));
        }
        else {
            this.defineAttributeFunction(attribudeId, [], () => value);
        }
        return this;
    }
    defineAttributeFunction(attribudeId, deps, fcn) {
        alwaysAssertThat(!A[attribudeId.name], () => {
            const mapping = {
                Required: 'defineRequiredIfVisible',
                Visible: 'defineVisibility',
                Label: 'defineLabel',
                InfoText: 'defineInfoText'
            };
            return `Please use specialised function ${mapping[attribudeId.name]} to define ${attribudeId.name}`;
        });
        this.addDependencies(deps, this.property, { [attribudeId.name]: true });
        this.property.defineAttribute({
            id: attribudeId,
            dependencies: deps,
            getValue: fcn
        });
    }
    defineVisibility(isVisible) {
        if (isVisible === false) {
            this.defineVisibilityFunction([], () => isVisible);
        }
        else if (isVisible instanceof Function) {
            this.defineVisibilityFunction([], () => isVisible(this.property));
        }
        return this;
    }
    defineVisibility1(dep1, isVisible) {
        this.defineVisibilityFunction([dep1], () => isVisible(this.property, dep1));
        return this;
    }
    defineVisibility2(dep1, dep2, isVisible) {
        this.defineVisibilityFunction([dep1, dep2], () => isVisible(this.property, dep1, dep2));
        return this;
    }
    defineVisibilityFunction(deps, fcn) {
        this.addDependencies(deps, this.property, { visible: true });
        this.property.defineVisibility({
            id: A.Visible,
            dependencies: deps,
            getValue: fcn
        });
    }
    defineRequiredIfVisible(required) {
        if (required === true) {
            this.defineRequiredIfVisibleFunction([], () => required);
        }
        else if (required instanceof Function) {
            this.defineRequiredIfVisibleFunction([], () => required(this.property));
        }
        return this;
    }
    defineRequiredIfVisibleFunction(deps, fcn) {
        this.addDependencies(deps, this.property, { required: true });
        this.property.defineRequiredIfVisible({
            id: A.Required,
            dependencies: deps,
            getValue: fcn
        });
        this.property.addScalarValidator(this.notEmptyIfRequiredValidator);
    }
    defineLabelAndPlaceholder(text) {
        this.defineLabel(text);
        return this.definePlaceholder(text);
    }
    defineLabel(text, textInterpreter) {
        this.property.defineLabel(this.interpreteText(text, textInterpreter));
        return this;
    }
    definePlaceholder(placeholder) {
        this.property.definePlaceholder(placeholder);
        return this;
    }
    defineInfoText(text, textInterpreter) {
        this.property.defineInfoText(this.interpreteText(text, textInterpreter));
        return this;
    }
    interpreteText(text, textInterpreter) {
        return text;
    }
    onUpdated(updated) {
        this.property.registerValueChangedListener({
            updated: () => updated(this.property)
        });
        return this;
    }
    setToInitialValueOnOtherPropertyChanged(...otherProperties) {
        alwaysAssertThat(!this.property.isReadOnly(), () => `${this.property.id}: Can only set properties to initial value, that are not read only`);
        return this;
    }
}
